@@ -18,15 +18,6 @@
  *
  */
 
-/*
- * CHANGES:
- *  2001/01/17: menno: Added frequency cut off filter.
- *  2001/02/28: menno: Added Temporal Noise Shaping.
- *  2001/03/05: menno: Added Long Term Prediction.
- *  2001/05/01: menno: Added backward prediction.
- *
- */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -42,8 +33,6 @@
 #include "util.h"
 #include "huffman.h"
 #include "tns.h"
-#include "ltp.h"
-#include "backpred.h"
 #include "version.h"
 
 #if FAAC_RELEASE
@@ -67,7 +56,6 @@ static SR_INFO srInfo[12+1];
 // default bandwidth/samplerate ratio
 static const double bwfac = 0.42;
 
-
 int FAACAPI faacEncGetVersion( char **faac_id_string,
 			      				char **faac_copyright_string)
 {
@@ -125,7 +113,8 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hpEncoder,
                                     faacEncConfigurationPtr config)
 {
     faacEncStruct* hEncoder = (faacEncStruct*)hpEncoder;
-	int i;
+    int i;
+    int maxqual = hEncoder->config.outputFormat ? MAXQUALADTS : MAXQUAL;
 
     hEncoder->config.allowMidside = config->allowMidside;
     hEncoder->config.useLfe = config->useLfe;
@@ -151,20 +140,15 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hpEncoder,
             break;
     }
 
-    /* No SSR supported for now */
-    if (hEncoder->config.aacObjectType == SSR)
-        return 0;
-
-    /* LTP only with MPEG4 */
-    if ((hEncoder->config.aacObjectType == LTP) && (hEncoder->config.mpegVersion != MPEG4))
+    if (hEncoder->config.aacObjectType != LOW)
         return 0;
 
     /* Re-init TNS for new profile */
     TnsInit(hEncoder);
 
     /* Check for correct bitrate */
-    if (config->bitRate > MaxBitrate(hEncoder->sampleRate))
-		return 0;
+    if (config->bitRate > (MaxBitrate(hEncoder->sampleRate) / hEncoder->numChannels))
+        config->bitRate = MaxBitrate(hEncoder->sampleRate) / hEncoder->numChannels;
 #if 0
     if (config->bitRate < MinBitrate())
         return 0;
@@ -235,7 +219,7 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hpEncoder,
 			config->bandWidth = f1;
 
 #ifndef DRM
-        config->bandWidth = config->bitRate * hEncoder->sampleRate * bwfac / 60000;
+        config->bandWidth = (double)config->bitRate * hEncoder->sampleRate * bwfac / 60000.0;
 #endif
     }
 
@@ -254,15 +238,19 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hpEncoder,
     if (hEncoder->config.bandWidth > (hEncoder->sampleRate / 2))
 		hEncoder->config.bandWidth = hEncoder->sampleRate / 2;
 
-    if (config->quantqual > 500)
-		config->quantqual = 500;
-    if (config->quantqual < 10)
-		config->quantqual = 10;
+    if (config->quantqual > maxqual)
+        config->quantqual = maxqual;
+    if (config->quantqual < MINQUAL)
+        config->quantqual = MINQUAL;
 
     hEncoder->config.quantqual = config->quantqual;
 
     /* set quantization quality */
     hEncoder->aacquantCfg.quality = config->quantqual;
+    BandLimit(&hEncoder->config.bandWidth,
+              hEncoder->sampleRate,
+              hEncoder->srInfo,
+              &hEncoder->aacquantCfg);
 
     // reset psymodel
     hEncoder->psymodel->PsyEnd(&hEncoder->gpsyInfo, hEncoder->psyInfo, hEncoder->numChannels);
@@ -296,7 +284,7 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
 	return NULL;
 
     *inputSamples = FRAME_LEN*numChannels;
-    *maxOutputBytes = (6144/8)*numChannels;
+    *maxOutputBytes = ADTS_FRAMESIZE;
 
 #ifdef DRM
     *maxOutputBytes += 1; /* for CRC */
@@ -318,13 +306,13 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
     hEncoder->config.name = libfaacName;
     hEncoder->config.copyright = libCopyright;
     hEncoder->config.mpegVersion = MPEG4;
-    hEncoder->config.aacObjectType = LTP;
+    hEncoder->config.aacObjectType = LOW;
     hEncoder->config.allowMidside = 1;
     hEncoder->config.useLfe = 1;
     hEncoder->config.useTns = 0;
     hEncoder->config.bitRate = 0; /* default bitrate / channel */
     hEncoder->config.bandWidth = bwfac * hEncoder->sampleRate;
-    hEncoder->config.quantqual = 100;
+    hEncoder->config.quantqual = DEFQUAL;
     hEncoder->config.psymodellist = (psymodellist_t *)psymodellist;
     hEncoder->config.psymodelidx = 0;
     hEncoder->psymodel =
@@ -334,13 +322,8 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
 	/* default channel map is straight-through */
 	for( channel = 0; channel < MAX_CHANNELS; channel++ )
 		hEncoder->config.channel_map[channel] = channel;
-	
-    /*
-        by default we have to be compatible with all previous software
-        which assumes that we will generate ADTS
-        /AV
-    */
-    hEncoder->config.outputFormat = 1;
+
+    hEncoder->config.outputFormat = ADTS_STREAM;
 
     /*
         be compatible with software which assumes 24bit in 32bit PCM
@@ -355,17 +338,12 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
         hEncoder->coderInfo[channel].prev_window_shape = SINE_WINDOW;
         hEncoder->coderInfo[channel].window_shape = SINE_WINDOW;
         hEncoder->coderInfo[channel].block_type = ONLY_LONG_WINDOW;
-        hEncoder->coderInfo[channel].num_window_groups = 1;
-        hEncoder->coderInfo[channel].window_group_length[0] = 1;
-
-        /* FIXME: Use sr_idx here */
-        hEncoder->coderInfo[channel].max_pred_sfb = GetMaxPredSfb(hEncoder->sampleRateIdx);
+        hEncoder->coderInfo[channel].groups.n = 1;
+        hEncoder->coderInfo[channel].groups.len[0] = 1;
 
         hEncoder->sampleBuff[channel] = NULL;
         hEncoder->nextSampleBuff[channel] = NULL;
         hEncoder->next2SampleBuff[channel] = NULL;
-        hEncoder->ltpTimeBuff[channel] = (double*)AllocMemory(2*BLOCK_LEN_LONG*sizeof(double));
-        SetMemory(hEncoder->ltpTimeBuff[channel], 0, 2*BLOCK_LEN_LONG*sizeof(double));
     }
 
     /* Initialize coder functions */
@@ -380,10 +358,6 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
 
     TnsInit(hEncoder);
 
-    LtpInit(hEncoder);
-
-    PredInit(hEncoder);
-
     AACQuantizeInit(hEncoder->coderInfo, hEncoder->numChannels,
 		    &(hEncoder->aacquantCfg));
 
@@ -405,8 +379,6 @@ int FAACAPI faacEncClose(faacEncHandle hpEncoder)
 
     FilterBankEnd(hEncoder);
 
-    LtpEnd(hEncoder);
-
     AACQuantizeEnd(hEncoder->coderInfo, hEncoder->numChannels,
 			&(hEncoder->aacquantCfg));
 
@@ -417,8 +389,6 @@ int FAACAPI faacEncClose(faacEncHandle hpEncoder)
     /* Free remaining buffer memory */
     for (channel = 0; channel < hEncoder->numChannels; channel++) 
 	{
-		if (hEncoder->ltpTimeBuff[channel])
-			FreeMemory(hEncoder->ltpTimeBuff[channel]);
 		if (hEncoder->sampleBuff[channel])
 			FreeMemory(hEncoder->sampleBuff[channel]);
 		if (hEncoder->nextSampleBuff[channel])
@@ -448,8 +418,6 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
     int sb, frameBytes;
     unsigned int offset;
     BitStream *bitStream; /* bitstream used for writing the frame to */
-    TnsInfo *tnsInfo_for_LTP;
-    TnsInfo *tnsDecInfo;
 #ifdef DRM
     int desbits, diff;
     double fix;
@@ -459,14 +427,12 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
     ChannelInfo *channelInfo = hEncoder->channelInfo;
     CoderInfo *coderInfo = hEncoder->coderInfo;
     unsigned int numChannels = hEncoder->numChannels;
-    unsigned int sampleRate = hEncoder->sampleRate;
-    unsigned int aacObjectType = hEncoder->config.aacObjectType;
-    unsigned int mpegVersion = hEncoder->config.mpegVersion;
     unsigned int useLfe = hEncoder->config.useLfe;
     unsigned int useTns = hEncoder->config.useTns;
     unsigned int allowMidside = hEncoder->config.allowMidside;
     unsigned int bandWidth = hEncoder->config.bandWidth;
     unsigned int shortctl = hEncoder->config.shortctl;
+    int maxqual = hEncoder->config.outputFormat ? MAXQUALADTS : MAXQUAL;
 
     /* Increase frame number */
     hEncoder->frameNum++;
@@ -487,17 +453,6 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
 	{
 		double *tmp;
 
-        if (hEncoder->sampleBuff[channel]) {
-            for(i = 0; i < FRAME_LEN; i++) {
-                hEncoder->ltpTimeBuff[channel][i] = hEncoder->sampleBuff[channel][i];
-            }
-        }
-        if (hEncoder->nextSampleBuff[channel]) {
-            for(i = 0; i < FRAME_LEN; i++) {
-                hEncoder->ltpTimeBuff[channel][FRAME_LEN + i] =
-						hEncoder->nextSampleBuff[channel][i];
-            }
-        }
 
 		if (!hEncoder->sampleBuff[channel])
 			hEncoder->sampleBuff[channel] = (double*)AllocMemory(FRAME_LEN*sizeof(double));
@@ -590,7 +545,7 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
     hEncoder->psymodel->PsyCalculate(channelInfo, &hEncoder->gpsyInfo, hEncoder->psyInfo,
         hEncoder->srInfo->cb_width_long, hEncoder->srInfo->num_cb_long,
         hEncoder->srInfo->cb_width_short,
-        hEncoder->srInfo->num_cb_short, numChannels);
+        hEncoder->srInfo->num_cb_short, numChannels, (double)hEncoder->aacquantCfg.quality / DEFQUAL);
 
     hEncoder->psymodel->BlockSwitch(coderInfo, hEncoder->psyInfo, numChannels);
 
@@ -612,24 +567,12 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
 
     /* AAC Filterbank, MDCT with overlap and add */
     for (channel = 0; channel < numChannels; channel++) {
-        int k;
-
         FilterBank(hEncoder,
             &coderInfo[channel],
             hEncoder->sampleBuff[channel],
             hEncoder->freqBuff[channel],
             hEncoder->overlapBuff[channel],
             MOVERLAPPED);
-
-        if (coderInfo[channel].block_type == ONLY_SHORT_WINDOW) {
-            for (k = 0; k < 8; k++) {
-                specFilter(hEncoder->freqBuff[channel]+k*BLOCK_LEN_SHORT,
-						sampleRate, bandWidth, BLOCK_LEN_SHORT);
-            }
-        } else {
-            specFilter(hEncoder->freqBuff[channel], sampleRate,
-					bandWidth, BLOCK_LEN_LONG);
-        }
     }
 
     /* TMP: Build sfb offset table and other stuff */
@@ -637,38 +580,36 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
         channelInfo[channel].msInfo.is_present = 0;
 
         if (coderInfo[channel].block_type == ONLY_SHORT_WINDOW) {
-			coderInfo[channel].max_sfb = hEncoder->srInfo->num_cb_short;
-            coderInfo[channel].nr_of_sfb = hEncoder->srInfo->num_cb_short;
+            coderInfo[channel].sfbn = hEncoder->aacquantCfg.max_cbs;
 
-            coderInfo[channel].num_window_groups = 1;
-            coderInfo[channel].window_group_length[0] = 8;
-            coderInfo[channel].window_group_length[1] = 0;
-            coderInfo[channel].window_group_length[2] = 0;
-            coderInfo[channel].window_group_length[3] = 0;
-            coderInfo[channel].window_group_length[4] = 0;
-            coderInfo[channel].window_group_length[5] = 0;
-            coderInfo[channel].window_group_length[6] = 0;
-            coderInfo[channel].window_group_length[7] = 0;
+            coderInfo[channel].groups.n = 1;
+            coderInfo[channel].groups.len[0] = 8;
+            coderInfo[channel].groups.len[1] = 0;
+            coderInfo[channel].groups.len[2] = 0;
+            coderInfo[channel].groups.len[3] = 0;
+            coderInfo[channel].groups.len[4] = 0;
+            coderInfo[channel].groups.len[5] = 0;
+            coderInfo[channel].groups.len[6] = 0;
+            coderInfo[channel].groups.len[7] = 0;
 
             offset = 0;
-            for (sb = 0; sb < coderInfo[channel].nr_of_sfb; sb++) {
+            for (sb = 0; sb < coderInfo[channel].sfbn; sb++) {
                 coderInfo[channel].sfb_offset[sb] = offset;
                 offset += hEncoder->srInfo->cb_width_short[sb];
             }
-            coderInfo[channel].sfb_offset[coderInfo[channel].nr_of_sfb] = offset;
+            coderInfo[channel].sfb_offset[sb] = offset;
         } else {
-            coderInfo[channel].max_sfb = hEncoder->srInfo->num_cb_long;
-            coderInfo[channel].nr_of_sfb = hEncoder->srInfo->num_cb_long;
+            coderInfo[channel].sfbn = hEncoder->aacquantCfg.max_cbl;
 
-            coderInfo[channel].num_window_groups = 1;
-            coderInfo[channel].window_group_length[0] = 1;
+            coderInfo[channel].groups.n = 1;
+            coderInfo[channel].groups.len[0] = 1;
 
             offset = 0;
-            for (sb = 0; sb < coderInfo[channel].nr_of_sfb; sb++) {
+            for (sb = 0; sb < coderInfo[channel].sfbn; sb++) {
                 coderInfo[channel].sfb_offset[sb] = offset;
                 offset += hEncoder->srInfo->cb_width_long[sb];
             }
-            coderInfo[channel].sfb_offset[coderInfo[channel].nr_of_sfb] = offset;
+            coderInfo[channel].sfb_offset[sb] = offset;
         }
     }
 
@@ -676,56 +617,16 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
     for (channel = 0; channel < numChannels; channel++) {
         if ((!channelInfo[channel].lfe) && (useTns)) {
             TnsEncode(&(coderInfo[channel].tnsInfo),
-					coderInfo[channel].max_sfb,
-					coderInfo[channel].max_sfb,
-					coderInfo[channel].block_type,
-					coderInfo[channel].sfb_offset,
-					hEncoder->freqBuff[channel]);
+                      coderInfo[channel].sfbn,
+                      coderInfo[channel].sfbn,
+                      coderInfo[channel].block_type,
+                      coderInfo[channel].sfb_offset,
+                      hEncoder->freqBuff[channel]);
         } else {
             coderInfo[channel].tnsInfo.tnsDataPresent = 0;      /* TNS not used for LFE */
         }
     }
 
-    for(channel = 0; channel < numChannels; channel++)
-    {
-        if((coderInfo[channel].tnsInfo.tnsDataPresent != 0) && (useTns))
-            tnsInfo_for_LTP = &(coderInfo[channel].tnsInfo);
-        else
-            tnsInfo_for_LTP = NULL;
-
-        if(channelInfo[channel].present && (!channelInfo[channel].lfe) &&
-            (coderInfo[channel].block_type != ONLY_SHORT_WINDOW) &&
-            (mpegVersion == MPEG4) && (aacObjectType == LTP))
-        {
-            LtpEncode(hEncoder,
-					&coderInfo[channel],
-					&(coderInfo[channel].ltpInfo),
-					tnsInfo_for_LTP,
-					hEncoder->freqBuff[channel],
-					hEncoder->ltpTimeBuff[channel]);
-        } else {
-            coderInfo[channel].ltpInfo.global_pred_flag = 0;
-        }
-    }
-
-    for(channel = 0; channel < numChannels; channel++)
-    {
-        if ((aacObjectType == MAIN) && (!channelInfo[channel].lfe)) {
-            int numPredBands = min(coderInfo[channel].max_pred_sfb, coderInfo[channel].nr_of_sfb);
-            PredCalcPrediction(hEncoder->freqBuff[channel],
-					coderInfo[channel].requantFreq,
-					coderInfo[channel].block_type,
-					numPredBands,
-					(coderInfo[channel].block_type==ONLY_SHORT_WINDOW)?
-					hEncoder->srInfo->cb_width_short:hEncoder->srInfo->cb_width_long,
-					coderInfo,
-					channelInfo,
-					channel);
-        } else {
-            coderInfo[channel].pred_global_flag = 0;
-        }
-    }
-
     for (channel = 0; channel < numChannels; channel++) {
 		if (coderInfo[channel].block_type == ONLY_SHORT_WINDOW) {
 			SortForGrouping(&coderInfo[channel],
@@ -737,7 +638,7 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
       // reduce LFE bandwidth
 		if (!channelInfo[channel].cpe && channelInfo[channel].lfe)
 		{
-			coderInfo[channel].nr_of_sfb = coderInfo[channel].max_sfb = 3;
+                    coderInfo[channel].sfbn = 3;
 		}
 	}
 
@@ -810,57 +711,15 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
 			cil = &coderInfo[channel];
 			cir = &coderInfo[channelInfo[channel].paired_ch];
 
-			cil->max_sfb = cir->max_sfb = max(cil->max_sfb, cir->max_sfb);
-			cil->nr_of_sfb = cir->nr_of_sfb = cil->max_sfb;
+                        cil->sfbn = cir->sfbn = max(cil->sfbn, cir->sfbn);
 		}
     }
-
-    MSReconstruct(coderInfo, channelInfo, numChannels);
-
-    for (channel = 0; channel < numChannels; channel++)
-    {
-        /* If short window, reconstruction not needed for prediction */
-        if ((coderInfo[channel].block_type == ONLY_SHORT_WINDOW)) {
-            int sind;
-            for (sind = 0; sind < BLOCK_LEN_LONG; sind++) {
-				coderInfo[channel].requantFreq[sind] = 0.0;
-            }
-        } else {
-
-            if((coderInfo[channel].tnsInfo.tnsDataPresent != 0) && (useTns))
-                tnsDecInfo = &(coderInfo[channel].tnsInfo);
-            else
-                tnsDecInfo = NULL;
-
-            if ((!channelInfo[channel].lfe) && (aacObjectType == LTP)) {  /* no reconstruction needed for LFE channel*/
-
-                LtpReconstruct(&coderInfo[channel], &(coderInfo[channel].ltpInfo),
-						coderInfo[channel].requantFreq);
-
-                if(tnsDecInfo != NULL)
-                    TnsDecodeFilterOnly(&(coderInfo[channel].tnsInfo), coderInfo[channel].nr_of_sfb,
-							coderInfo[channel].max_sfb, coderInfo[channel].block_type,
-							coderInfo[channel].sfb_offset, coderInfo[channel].requantFreq);
-
-                IFilterBank(hEncoder, &coderInfo[channel],
-						coderInfo[channel].requantFreq,
-						coderInfo[channel].ltpInfo.time_buffer,
-						coderInfo[channel].ltpInfo.ltp_overlap_buffer,
-						MOVERLAPPED);
-
-                LtpUpdate(&(coderInfo[channel].ltpInfo),
-						coderInfo[channel].ltpInfo.time_buffer,
-						coderInfo[channel].ltpInfo.ltp_overlap_buffer,
-						BLOCK_LEN_LONG);
-            }
-        }
-    }
-
 #ifndef DRM
     /* Write the AAC bitstream */
     bitStream = OpenBitStream(bufferSize, outputBuffer);
 
-    WriteBitstream(hEncoder, coderInfo, channelInfo, bitStream, numChannels);
+    if (WriteBitstream(hEncoder, coderInfo, channelInfo, bitStream, numChannels) < 0)
+        return -1;
 
     /* Close the bitstream and return the number of bytes written */
     frameBytes = CloseBitStream(bitStream);
@@ -882,8 +741,8 @@ int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
 		if (((diff > 0) && (fix > 0.0)) || ((diff < 0) && (fix < 0.0)))
 		{
 			hEncoder->aacquantCfg.quality *= (1.0 - fix);
-			if (hEncoder->aacquantCfg.quality > 300)
-				hEncoder->aacquantCfg.quality = 300;
+                        if (hEncoder->aacquantCfg.quality > maxqual)
+                            hEncoder->aacquantCfg.quality = maxqual;
             if (hEncoder->aacquantCfg.quality < 50)
                 hEncoder->aacquantCfg.quality = 50;
 		}
